#!/usr/bin/python

"""Install libTAU on a Mac system"""

from optparse import OptionParser
import os
import shutil
import glob
import struct
import subprocess

def is_32bit():
    return struct.calcsize("P") == 4

def is_arm64():
    return os.uname()[4] == 'arm64'

def get_options():
    parser = OptionParser("""%prog [options] destination

Install the libTAU headers and library on a Mac system.
""")
    parser.add_option("--universal", action="store_true", default=False,
                      help="Install universal binaries")
    opts, args = parser.parse_args()
    if len(args) != 1:
        parser.error("incorrect number of arguments")
    return opts.universal, args[0]

def copy_includes(incdir):
    if not os.path.exists(incdir):
        os.makedirs(incdir)
    for f in glob.glob("include/*.h"):
        shutil.copy(f, incdir)

def get_darwin_version():
    return int(os.uname()[2].split('.')[0])

def copy_libs(libdir, universal):
    libname = 'libTAU.1.dylib'
    # The 10.6 library *should* also work on newer Macs up to 10.8
    lib32 = os.path.join('lib', 'Mac10.6.i386', libname)
    lib64 = os.path.join('lib', 'Mac10.6.x86_64', libname)
    libarm64 = os.path.join('lib', 'Mac11.arm64', libname)
    # For 10.9 or later (Darwin kernel 13 or later) use the library
    # built with clang++ rather than g++
    if get_darwin_version() >= 13:
        lib32 = os.path.join('lib', 'Mac10.9.i386', libname)
        lib64 = os.path.join('lib', 'Mac10.9.x86_64', libname)
    libinst = os.path.join(libdir, libname)
    if not os.path.exists(libdir):
        os.makedirs(libdir)
    if universal:
        subprocess.check_call(['lipo', '-create', '-arch', 'i386', lib32,
                               '-arch', 'x86_64', lib64, '-output', libinst])
    elif is_arm64():
        shutil.copy(libarm64, libdir)
    elif is_32bit():
        shutil.copy(lib32, libdir)
    else:
        shutil.copy(lib64, libdir)
    subprocess.check_call(['install_name_tool', '-id', libinst, libinst])
    os.symlink(libname, os.path.join(libdir, 'libTAU.dylib'))

def main():
    universal, dest = get_options()
    copy_includes(os.path.join(dest, 'include', 'libTAU'))
    copy_libs(os.path.join(dest, 'lib'), universal)

if __name__ == '__main__':
    main()
